searchNodes=[{"ref":"Phoenix.LiveComponent.html","title":"Phoenix.LiveComponent","type":"behaviour","doc":"Components are a mechanism to compartmentalize state, markup, and events in LiveView. Components are defined by using Phoenix.LiveComponent and are used by calling Phoenix.LiveView.live_component/3 in a parent LiveView. Components run inside the LiveView process, but may have their own state and event handling. The simplest component only needs to define a render function: defmodule HeroComponent do use Phoenix.LiveComponent def render(assigns) do ~L&quot;&quot;&quot; &lt;div class=&quot;hero&quot;&gt;&lt;%= @content %&gt;&lt;/div&gt; &quot;&quot;&quot; end end When use Phoenix.LiveComponent is used, all functions in Phoenix.LiveView are imported. A component can be invoked as: &lt;%= live_component @socket, HeroComponent, content: @content %&gt; Components come in two shapes, stateless or stateful. The component above is a stateless component. Of course, the component above is not any different compared to a regular function. However, as we will see, components do provide their own exclusive feature set. Stateless components life-cycle When live_component is called, the following callbacks will be invoked in the component: mount(socket) -&gt; update(assigns, socket) -&gt; render(assigns) First mount/1 is called only with the socket. mount/1 can be used to set any initial state. Then update/2 is invoked with all of the assigns given to live_component/2. The default implementation of update/2 simply merges all assigns into the socket. Then, after the component is updated, render/1 is called with all assigns. A stateless component is always mounted, updated, and rendered whenever the parent template changes. That&#39;s why they are stateless: no state is kept after the component. However, any component can be made stateful by passing an :id assign. Stateful components life-cycle A stateful component is a component that receives an :id on live_component/2: &lt;%= live_component @socket, HeroComponent, id: :hero, content: @content %&gt; Stateful components are identified by the component module and their ID. Therefore, two different component modules with the same ID are different components. This means we can often tie the component ID to some application based ID: &lt;%= live_component @socket, UserComponent, id: @user.id, user: @user %&gt; Also note the given :id is not necessarily used as the DOM ID. If you want to set a DOM ID, it is your responsibility to set it when rendering: defmodule UserComponent do use Phoenix.LiveComponent def render(assigns) do ~L&quot;&quot;&quot; &lt;div id=&quot;user-&lt;%= @id %&gt;&quot; class=&quot;user&quot;&gt;&lt;%= @user.name %&gt;&lt;/div&gt; &quot;&quot;&quot; end end In stateful components, mount/1 is called only once, when the component is first rendered. Then for each rendering, the optional preload/1 and update/2 callbacks are called before render/1. Targeting Component Events Stateful components can also implement the handle_event/3 callback that works exactly the same as in LiveView. For a client event to reach a component, the tag must be annotated with a phx-target annotation which must be a query selector to an element inside the component. For example, if the UserComponent above is started with the :id of 13, it will have the DOM ID of user-13. Using a query selector, we can sent an event to it with: &lt;a href=&quot;#&quot; phx-click=&quot;say_hello&quot; phx-target=&quot;#user-13&quot;&gt; Say hello! &lt;/a&gt; Then handle_event/3 will be called by with the &quot;say_hello&quot; event. When handle_event/3 is called for a component, only the diff of the component is sent to the client, making them extremely efficient. Any valid query selector for phx-target is supported, provided the matched nodes are children of a LiveView or LiveComponent, for example to send the close event to multiple components: &lt;a href=&quot;#&quot; phx-click=&quot;close&quot; phx-target=&quot;#modal, #sidebar&quot;&gt; Dismiss &lt;/a&gt; Preloading and update Every time a stateful component is rendered, both preload/1 and update/2 is called. To understand why both callbacks are necessary, imagine that you implement a component and the component needs to load some state from the database. For example: &lt;%= live_component @socket, UserComponent, id: user_id %&gt; A possible implementation would be to load the user on the update/2 callback: def update(assigns, socket) do user = Repo.get! User, assigns.id {:ok, assign(socket, :user, user)} end However, the issue with said approach is that, if you are rendering multiple user components in the same page, you have a N+1 query problem. The preload/1 callback helps address this problem as it is invoked with a list of assigns for all components of the same type. For example, instead of implementing update/2 as above, one could implement: def preload(list_of_assigns) do list_of_ids = Enum.map(list_of_assigns, &amp; &amp;1.id) users = from(u in User, where: u.id in ^list_of_ids, select: {u.id, u}) |&gt; Repo.all() |&gt; Map.new() Enum.map(list_of_assigns, fn assigns -&gt; Map.put(assigns, :user, users[assigns.id]) end) end Now only a single query to the database will be made. In fact, the preloading algorithm is a breadth-first tree traversal, which means that even for nested components, the amount of queries are kept to a minimum. Finally, note that preload/1 must return an updated list_of_assigns, keeping the assigns in the same order as they were given. Managing state Now that we have learned how to define and use components, as well as how to use preload/1 as a data loading optimization, it is important to talk about how to manage state in components. Generally speaking, you want to avoid both the parent LiveView and the LiveComponent working on two different copies of the state. Instead, you should assume only one of them to be the source of truth. Let&#39;s discuss these approaches in detail. Imagine that the scenario we will explore is that we have a LiveView representing a board, where each card in the board is a separate component. Each card has a form that allows to update the form title directly in the component. We will see how to organize the data flow keeping either the view or the component as the source of truth. LiveView as the source of truth If the LiveView is the source of truth, the LiveView will be responsible for fetching all of the cards in a board. Then it will call live_component/2 for each card, passing the card struct as argument to CardComponent: &lt;%= for card &lt;- @cards do %&gt; &lt;%= live_component CardComponent, card: card, board_id: @id %&gt; &lt;% end %&gt; Now, when the user submits a form inside the CardComponent to update the card, CardComponent.handle_event/3 will be triggered. However, if the update succeeds, you must not change the card struct inside the component. If you do so, the card struct in the component will get out of sync with the LiveView. Since the LiveView is the source of truth, we should instead tell the LiveView the card was updated. Luckily, because the component and the view run in the same process, sending a message from the component to the parent LiveView is as simple as sending a message to self: defmodule CardComponent do ... def handle_event(&quot;update_title&quot;, %{&quot;title&quot; =&gt; title}, socket) do send self(), {:updated_card, %{socket.assigns.card | title: title}} {:noreply, socket} end end The LiveView can receive this event using handle_info: defmodule BoardView do ... def handle_info({:updated_card, card}, socket) do # update the list of cards in the socket {:noreply, updated_socket} end end As the list of cards in the parent socket was updated, the parent will be re-rendered, sending the updated card to the component. So in the end, the component does get the updated card, but always driven from the parent. Alternatively, instead of having the component directly send a message to the parent, the component could broadcast the update using Phoenix.PubSub. Such as: defmodule CardComponent do ... def handle_event(&quot;update_title&quot;, %{&quot;title&quot; =&gt; title}, socket) do message = {:updated_card, %{socket.assigns.card | title: title}} Phoenix.PubSub.broadcast(MyApp.PubSub, board_topic(socket), message) {:noreply, socket} end defp board_topic(socket) do &quot;board:&quot; &lt;&gt; socket.assigns.board_id end end As long as the parent LiveView subscribes to the &quot;board:ID&quot; topic, it will receive updates. The advantage of using PubSub is that we get distributed updates out of the box. Now if any user connected to the board changes a card, all other users will see the change. LiveComponent as the source of truth If the component is the source of truth, then the LiveView must no longer fetch all of the cards structs from the database. Instead, the view must only fetch all of the card ids and render the component only by passing the IDs: &lt;%= for card_id &lt;- @card_ids do %&gt; &lt;%= live_component CardComponent, card_id: card_id, board_id: @id %&gt; &lt;% end %&gt; Now, each CardComponent loads their own card. Of course, doing so per card would be expensive and lead to N queries, where N is the number of components, so we must use the preload/1 callback to make it efficient. Once all card components are started, they can fully manage each card as a whole, without concerning themselves with the parent LiveView. However, note that components do not have a handle_info/2 callback. Therefore, if you want to track distributed changes on a card, you must have the parent LiveView receive those events and redirect them to the appropriate card. For example, assuming card updates are sent to the &quot;board:ID&quot; topic, and that the board LiveView is subscribed to said topic, one could do: def handle_info({:updated_card, card}, socket) do send_update CardComponent, id: card.id, board_id: socket.assigns.id {:noreply, socket} end With send_update, the CardComponent given by id will be invoked, triggering both preload and update callbacks, which will load the most up to date data from the database. Live component blocks When live_component is invoked, it is also possible to pass a do/end block: &lt;%= live_component @socket, GridComponent, entries: @entries do %&gt; New entry: &lt;%= @entry %&gt; &lt;% end %&gt; The do/end will be available as an anonymous function in an assign named @inner_content. The anonymous function must be invoked passing a new set of assigns that will be merged into the user assigns. For example, the grid component above could be implemented as: defmodule GridComponent do use Phoenix.LiveComponent def render(assigns) do ~L&quot;&quot;&quot; &lt;div class=&quot;grid&quot;&gt; &lt;%= for entry &lt;- @entries do %&gt; &lt;div class=&quot;column&quot;&gt; &lt;%= @inner_content.(entry: entry) %&gt; &lt;/div&gt; &lt;% end %&gt; &lt;/div&gt; &quot;&quot;&quot; end end Where the :entry assign was injected into the do/end block. The approach above is the preferred one when passing blocks to do/end. However, if you are outside of a .leex template and you want to invoke a component passing do/end blocks, you will have to explicitly handle the assigns by giving it a clause: live_component @socket, GridComponent, entries: @entries do new_assigns -&gt; &quot;New entry: &quot; &lt;&gt; new_assigns[:entry] end Live links and live redirects A template rendered inside a component can use live_link calls. The live_link is always handled by the parent LiveView, as components do not provide handle_params. live_redirect from inside a component is not currently supported. For such, you must send a message to the LiveView itself, as mentioned above, which may then redirect. Limitations Components must only contain HTML tags at their root. At least one HTML tag must be present. It is not possible to have components that render only text or text mixed with tags at the root. Another limitation of components is that they must always be change tracked. For example, if you render a component inside form_for, like this: &lt;%= form_for @changeset, &quot;#&quot;, fn f -&gt; %&gt; &lt;%= live_component @socket, SomeComponent, f: f %&gt; &lt;% end %&gt; The component ends up enclosed by the form markup, where LiveView cannot track it. In such cases, you may receive an error such as: ** (ArgumentError) cannot convert component SomeComponent to HTML. A component must always be returned directly as part of a LiveView template In this particular case, this can be addressed by using the form_for variant without anonymous functions: &lt;%= f = form_for @changeset, &quot;#&quot; %&gt; &lt;%= live_component @socket, SomeComponent, f: f %&gt; &lt;/form&gt; This issue can also happen with other helpers, such as content_tag: &lt;%= content_tag :div do %&gt; &lt;%= live_component @socket, SomeComponent, f: f %&gt; &lt;% end %&gt; In this case, the solution is to not use content_tag and rely on LiveEEx to build the markup."},{"ref":"Phoenix.LiveComponent.html#c:handle_event/3","title":"Phoenix.LiveComponent.handle_event/3","type":"callback","doc":""},{"ref":"Phoenix.LiveComponent.html#c:mount/1","title":"Phoenix.LiveComponent.mount/1","type":"callback","doc":""},{"ref":"Phoenix.LiveComponent.html#c:preload/1","title":"Phoenix.LiveComponent.preload/1","type":"callback","doc":""},{"ref":"Phoenix.LiveComponent.html#c:render/1","title":"Phoenix.LiveComponent.render/1","type":"callback","doc":""},{"ref":"Phoenix.LiveComponent.html#c:update/2","title":"Phoenix.LiveComponent.update/2","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html","title":"Phoenix.LiveView","type":"behaviour","doc":"LiveView provides rich, real-time user experiences with server-rendered HTML. LiveView programming model is declarative: instead of saying &quot;once event X happens, change Y on the page&quot;, events in LiveView are regular messages which may cause changes to its state. Once the state changes, LiveView will re-render the relevant parts of its HTML template and push it to the browser, which updates itself in the most efficient manner. This means developers write LiveView templates as any other server-rendered HTML and LiveView does the hard work of tracking changes and sending the relevant diffs to the browser. At the end of the day, a LiveView is nothing more than a process, that receives events as messages and updates its state. The state itself is nothing more than functional and immutable Elixir data structures. The events are either internal application messages (usually emitted by Phoenix.PubSub) or sent by the client/browser. LiveView provides many features that make it excellent to build rich, real-time user experiences: By building on top of Elixir processes and Phoenix.Channels, LiveView scales well vertically (from small to large instances) and horizontally (by adding more instances); LiveView is first rendered statically as part of regular HTTP requests, which provides quick times for &quot;First Meaningful Paint&quot; and also help search and indexing engines; LiveView performs diff tracking. If the LiveView state changes, it won&#39;t re-render the whole template, but only the parts affected by the changed state. This reduces latency and the amount of data sent over the wire; LiveView tracks static and dynamic contents. Any server-rendered HTML is made of static parts (i.e. that never change) and dynamic ones. On the first render, LiveView sends the static contents and in future updates only the modified dynamic contents are resent; (Coming soon) LiveView uses the Erlang Term Format to send messages to the client. This binary-based format is quite efficient on the server and uses less data over the wire; (Coming soon) LiveView includes a latency simulator, which allows you to simulate how your application behaves on increased latency and guides you to provide meaningful feedback to users while they wait for events to be processed; Furthermore, by keeping a persistent connection between client and server, LiveView applications can react faster to user events as there is less work to be done and less data to be sent compared to stateless requests that have to authenticate, decode, load, and encode data on every request. The flipside is that LiveView uses more memory on the server compared to stateless requests. Use cases There are many use cases where LiveView is an excellent fit right now: Handling of user interaction and inputs, buttons, and forms - such as input validation, dynamic forms, autocomplete, etc; Events and updates pushed by server - such as notifications, dashboards, etc; Page and data navigation - such as navigating between pages, pagination, etc can be built with LiveView but currently you will lose the back/forward button, and the ability to link to pages as you navigate. Support for pushState is on the roadmap; There are other cases that have limited support but will become first-class as we further develop LiveView: Transitions and loading states - the LiveView programming model provides a good foundation for transitions and loading states since any UI change done after a user action is undone once the server sends the update for said action. For example, it is relatively straight-forward to click a button that changes itself in a way that is automatically undone when the update arrives. This is especially important as user feedback when latency is involved. A complete feature set for modelling those states is coming in future versions; Optimistic UIs - once we add transitions and loading states, many of the building blocks necessary for building optimistic UIs will be part of LiveView, but since optimistic UIs are about doing work on the client while the server is unavailable, complete support for Optimistic UIs cannot be achieved without also writing JavaScript for the cases the server is not available. See &quot;JS Interop and client controlled DOM&quot; on how to integrate JS hooks; There are also use cases which are a bad fit for LiveView: Animations - animations, menus, and general events that do not need the server in the first place are a bad fit for LiveView, as they can be achieved purely with CSS and/or CSS transitions; Life-cycle A LiveView begins as a regular HTTP request and HTML response, and then upgrades to a stateful view on client connect, guaranteeing a regular HTML page even if JavaScript is disabled. Any time a stateful view changes or updates its socket assigns, it is automatically re-rendered and the updates are pushed to the client. You begin by rendering a LiveView from your router, controller, or view. When a view is first rendered, the mount/2 callback is invoked with the current session and the LiveView socket. The mount/2 callback wires up socket assigns necessary for rendering the view. After mounting, render/1 is invoked and the HTML is sent as a regular HTML response to the client. After rendering the static page, LiveView connects from the client where stateful views are spawned to push rendered updates to the browser, and receive client events via phx bindings. Just like the first rendering, mount/2 is invoked with the session, and socket state, where mount assigns values for rendering. However in the connected client case, a LiveView process is spawned on the server, pushes the result of render/1 to the client and continues on for the duration of the connection. If at any point during the stateful life-cycle a crash is encountered, or the client connection drops, the client gracefully reconnects to the server, calling mount/2 once again. Example First, a LiveView requires two callbacks: mount/2 and render/1: defmodule AppWeb.ThermostatLive do use Phoenix.LiveView def render(assigns) do ~L&quot;\\&quot;&quot; Current temperature: &lt;%= @temperature %&gt; &quot;\\&quot;&quot; end def mount(%{&quot;current_user_id&quot; =&gt; user_id}, socket) do temperature = Thermostat.get_user_reading(user_id) {:ok, assign(socket, :temperature, temperature)} end end The render/1 callback receives the socket.assigns and is responsible for returning rendered content. You can use Phoenix.LiveView.sigil_L/2 to inline LiveView templates. If you want to use Phoenix.HTML helpers, remember to use Phoenix.HTML at the top of your LiveView. A separate .leex HTML template can also be rendered within your render/1 callback by delegating to an existing Phoenix.View module in your application. For example: defmodule AppWeb.ThermostatLive do use Phoenix.LiveView def render(assigns) do Phoenix.View.render(AppWeb.PageView, &quot;page.html&quot;, assigns) end end With a LiveView defined, you first define the socket path in your endpoint, and point it to Phoenix.LiveView.Socket: defmodule AppWeb.Endpoint do use Phoenix.Endpoint socket &quot;/live&quot;, Phoenix.LiveView.Socket, websocket: [connect_info: [session: @session_options]] ... end Where @session_options are the options given to plug Plug.Session extracted to a module attribute. And configure its signing salt in the endpoint: config :my_app, AppWeb.Endpoint, ..., live_view: [signing_salt: ...] You can generate a secure, random signing salt with the mix phx.gen.secret 32 task. Next, decide where you want to use your LiveView. You can serve the LiveView directly from your router (recommended): defmodule AppWeb.Router do use Phoenix.Router import Phoenix.LiveView.Router scope &quot;/&quot;, AppWeb do live &quot;/thermostat&quot;, ThermostatLive end end You can also live_render from any template: &lt;h1&gt;Temperature Control&lt;/h1&gt; &lt;%= live_render(@conn, AppWeb.ThermostatLive) %&gt; Or you can live_render your view from any controller: defmodule AppWeb.ThermostatController do ... import Phoenix.LiveView.Controller def show(conn, %{&quot;id&quot; =&gt; id}) do live_render(conn, AppWeb.ThermostatLive) end end When a LiveView is rendered, all of the data currently stored in the connection session (see Plug.Conn.get_session/1) will be given to the LiveView. It is also possible to pass extra session information besides the one currently in the connection session to the LiveView, by passing a session parameter: # In the router live &quot;/thermostat&quot;, ThermostatLive, session: %{&quot;extra_token&quot; =&gt; &quot;foo&quot;} # In a view &lt;%= live_render(@conn, AppWeb.ThermostatLive, session: %{&quot;extra_token&quot; =&gt; &quot;foo&quot;}) %&gt; Notice the :session uses string keys as a reminder that session data is serialized and sent to the client. So you should always keep the data in the session to a minimum. I.e. instead of storing a User struct, you should store the &quot;user_id&quot; and load the User when the LiveView mounts. Once the LiveView is rendered, a regular HTML response is sent. Next, your client code connects to the server: import {Socket} from &quot;phoenix&quot; import LiveSocket from &quot;phoenix_live_view&quot; let csrfToken = document.querySelector(&quot;meta[name=&#39;csrf-token&#39;]&quot;).getAttribute(&quot;content&quot;); let liveSocket = new LiveSocket(&quot;/live&quot;, {params: {_csrf_token: csrfToken}}); liveSocket.connect() Note: Comprehensive JavaScript client usage is covered in a later section. After the client connects, mount/2 will be invoked inside a spawned LiveView process. At this point, you can use connected?/1 to conditionally perform stateful work, such as subscribing to pubsub topics, sending messages, etc. For example, you can periodically update a LiveView with a timer: defmodule DemoWeb.ThermostatLive do use Phoenix.LiveView ... def mount(%{&quot;current_user_id&quot; =&gt; user_id}, socket) do if connected?(socket), do: :timer.send_interval(30000, self(), :update) case Thermostat.get_user_reading(user_id) do {:ok, temperature} -&gt; {:ok, assign(socket, temperature: temperature, user_id: user_id)} {:error, reason} -&gt; {:error, reason} end end def handle_info(:update, socket) do {:ok, temperature} = Thermostat.get_reading(socket.assigns.user_id) {:noreply, assign(socket, :temperature, temperature)} end end We used connected?(socket) on mount to send our view a message every 30s if the socket is in a connected state. We receive :update in a handle_info just like a GenServer, and update our socket assigns. Whenever a socket&#39;s assigns change, render/1 is automatically invoked, and the updates are sent to the client. Assigns and LiveEEx Templates All of the data in a LiveView is stored in the socket as assigns. The assign/2 and assign/3 functions help store those values. Those values can be accessed in the LiveView as socket.assigns.name but they are most commonly accessed inside LiveView templates as @name. Phoenix.LiveView&#39;s built-in templates provided by the .leex extension or ~L sigil, stands for Live EEx. They are similar to regular .eex templates except they are designed to minimize the amount of data sent over the wire by splitting static from dynamic parts and also tracking changes. When you first render a .leex template, it will send all of the static and dynamic parts of the template to the client. After that, any change you do on the server will now send only the dynamic parts, and only if those parts have changed. The tracking of changes is done via assigns. Imagine this template: &lt;div id=&quot;user_&lt;%= @user.id %&gt;&quot;&gt; &lt;%= @user.name %&gt; &lt;/div&gt; If the @user assign changes, then LiveView will re-render only the @user.id and @user.name and send them to the browser. The change tracking also works when rendering other templates, as long as they are also .leex templates and as long as all assigns are passed to the child/inner template: &lt;%= render &quot;child_template.html&quot;, assigns %&gt; The assign tracking feature also implies that you MUST avoid performing direct operations in the template. For example, if you perform a database query in your template: &lt;%= for user &lt;- Repo.all(User) do %&gt; &lt;%= user.name %&gt; &lt;% end %&gt; Then Phoenix will never re-render the section above, even if the number of users in the database changes. Instead, you need to store the users as assigns in your LiveView before it renders the template: assign(socket, :users, Repo.all(User)) Generally speaking, data loading should never happen inside the template, regardless if you are using LiveView or not. The difference is that LiveView enforces those as best practices. Change tracking pitfalls Although change tracking can considerably reduce the amount of data sent over the wire, there are some pitfalls users should be aware of. First of all, change tracking can only track assigns directly. So for example, if you do something such as: &lt;%= @post.the_whole_content %&gt; If any of other field besides the_whole_content in @post change for any reason, the the_whole_content will be sent downstream. Although this is not generally a problem, if you have large fields that you don&#39;t want to resend or if you have one field in particular that changes all the time while others do not, you may want to track them as their own assign. Another limitation of changing tracking is that it does not work across regular function calls. For example, imagine the following template that renders a div: &lt;%= content_tag :div, id: &quot;&quot;user_#\\{@id}&quot; do %&gt; &lt;%= @name %&gt; &lt;%= @description %&gt; &lt;% end %&gt; LiveView knows nothing about content_tag, which means the whole div will be sent whenever any of the assigns change. This can be easily fixed by writing the HTML directly: &lt;div id=&quot;user_&lt;%= @id %&gt;&quot;&gt; &lt;%= @name %&gt; &lt;%= @description %&gt; &lt;/div&gt; Note though this concern does not apply to Elixir&#39;s constructs, such as if, case, for, and friends. LiveView always knows how to optimize across those. Bindings Phoenix supports DOM element bindings for client-server interaction. For example, to react to a click on a button, you would render the element: &lt;button phx-click=&quot;inc_temperature&quot;&gt;+&lt;/button&gt; Then on the server, all LiveView bindings are handled with the handle_event callback, for example: def handle_event(&quot;inc_temperature&quot;, _value, socket) do {:ok, new_temp} = Thermostat.inc_temperature(socket.assigns.id) {:noreply, assign(socket, :temperature, new_temp)} end BindingAttributes Paramsphx-value-* Click Eventsphx-click, phx-target Focus/Blur Eventsphx-blur, phx-focus, phx-target Form Eventsphx-change, phx-submit, phx-target, data-phx-error-for, phx-disable-with Key Eventsphx-keydown, phx-keyup, phx-target Rate Limitingphx-debounce, phx-throttle Custom DOM Patchingphx-update JS Interopphx-hook Click Events The phx-click binding is used to send click events to the server. When any client event, such as a phx-click click is pushed, the value sent to the server will be chosen with the following priority: Any number of optional phx-value- prefixed attributes, such as: &lt;div phx-click=&quot;inc&quot; phx-value-myvar1=&quot;val1&quot; phx-value-myvar2=&quot;val2&quot;&gt; will send the following map of params to the server: def handle_event(&quot;inc&quot;, %{&quot;myvar1&quot; =&gt; &quot;val1&quot;, &quot;myvar2&quot; =&gt; &quot;val2&quot;}, socket) do If the phx-value- prefix is used, the server payload will also contain a &quot;value&quot; if the element&#39;s value attribute exists. When receiving a map on the server, the payload will also contain metadata of the client event, containing all literal keys of the event object, such as a click event&#39;s clientX, a keydown event&#39;s keyCode, etc. Focus and Blur Events Focus and blur events may be bound to DOM elements that emit such events, using the phx-blur, and phx-focus bindings, for example: &lt;input name=&quot;email&quot; phx-focus=&quot;myfocus&quot; phx-blur=&quot;myblur&quot;/&gt; To detect when the page itself has received focus or blur, phx-window-focus and phx-window-blur may be specified. Like other bindings, phx-value-* can be provided on the bound element, and those values will be sent as part of the payload. For example: &lt;div class=&quot;container&quot; phx-window-focus=&quot;page-active&quot; phx-window-blur=&quot;page-inactive&quot; phx-value-page=&quot;123&quot;&gt; ... &lt;/div&gt; The following window level bindings are supported: phx-window-focus phx-window-blur phx-window-keydown phx-window-keyup Form Events To handle form changes and submissions, use the phx-change and phx-submit events. In general, it is preferred to handle input changes at the form level, where all form fields are passed to the LiveView&#39;s callback given any single input change. For example, to handle real-time form validation and saving, your template would use both phx_change and phx_submit bindings: &lt;%= f = form_for @changeset, &quot;#&quot;, [phx_change: :validate, phx_submit: :save] %&gt; &lt;%= label f, :username %&gt; &lt;%= text_input f, :username %&gt; &lt;%= error_tag f, :username %&gt; &lt;%= label f, :email %&gt; &lt;%= text_input f, :email %&gt; &lt;%= error_tag f, :email %&gt; &lt;%= submit &quot;Save&quot; %&gt; &lt;/form&gt; Next, your LiveView picks up the events in handle_event callbacks: def render(assigns) ... def mount(_session, socket) do {:ok, assign(socket, %{changeset: Accounts.change_user(%User{})})} end def handle_event(&quot;validate&quot;, %{&quot;user&quot; =&gt; params}, socket) do changeset = %User{} |&gt; Accounts.change_user(params) |&gt; Map.put(:action, :insert) {:noreply, assign(socket, changeset: changeset)} end def handle_event(&quot;save&quot;, %{&quot;user&quot; =&gt; user_params}, socket) do case Accounts.create_user(user_params) do {:ok, user} -&gt; {:stop, socket |&gt; put_flash(:info, &quot;user created&quot;) |&gt; redirect(to: Routes.user_path(AppWeb.Endpoint, AppWeb.User.ShowView, user))} {:error, %Ecto.Changeset{} = changeset} -&gt; {:noreply, assign(socket, changeset: changeset)} end end The validate callback simply updates the changeset based on all form input values, then assigns the new changeset to the socket. If the changeset changes, such as generating new errors, render/1 is invoked and the form is re-rendered. Likewise for phx-submit bindings, the same callback is invoked and persistence is attempted. On success, a :stop tuple is returned and the socket is annotated for redirect with Phoenix.LiveView.redirect/2 to the new user page, otherwise the socket assigns are updated with the errored changeset to be re-rendered for the client. Note: For proper form error tag updates, the error tag must specify which input it belongs to. This is accomplished with the data-phx-error-for attribute. For example, your AppWeb.ErrorHelpers may use this function: def error_tag(form, field) do Enum.map(Keyword.get_values(form.errors, field), fn error -&gt; content_tag(:span, translate_error(error), class: &quot;help-block&quot;, data: [phx_error_for: input_id(form, field)] ) end) end Number inputs Number inputs are a special case in LiveView forms. On programmatic updates, some browsers will clear invalid inputs. So LiveView will not send change events from the client when an input is invalid, instead allowing the browser&#39;s native validation UI to drive user interaction. Once the input becomes valid, change and submit events will be sent as normal. Password inputs Password inputs are also special cased in Phoenix.HTML. For security reasons, password field values are not reused when rendering a password input tag. This requires explicitly setting the :value in your markup, for example: &lt;%= password_input f, :password, value: input_value(f, :password) %&gt; &lt;%= password_input f, :password_confirmation, value: input_value(f, :password_confirmation) %&gt; &lt;%= error_tag f, :password %&gt; &lt;%= error_tag f, :password_confirmation %&gt; Key Events The onkeydown, and onkeyup events are supported via the phx-keydown, and phx-keyup bindings. When pushed, the value sent to the server will contain all the client event object&#39;s metadata. For example, pressing the Escape key looks like this: %{ &quot;altKey&quot; =&gt; false, &quot;charCode&quot; =&gt; 0, &quot;code&quot; =&gt; &quot;Escape&quot;, &quot;ctrlKey&quot; =&gt; false, &quot;key&quot; =&gt; &quot;Escape&quot;, &quot;keyCode&quot; =&gt; 27, &quot;location&quot; =&gt; 0, &quot;metaKey&quot; =&gt; false, &quot;repeat&quot; =&gt; false, &quot;shiftKey&quot; =&gt; false, &quot;which&quot; =&gt; 27 } By default, the bound element will be the event listener, but a window-level binding may be provided via phx-window-keydown, for example: def render(assigns) do ~L&quot;\\&quot;&quot; &lt;div id=&quot;thermostat&quot; phx-window-keyup=&quot;update_temp&quot;&gt; Current temperature: &lt;%= @temperature %&gt; &lt;/div&gt; &quot;\\&quot;&quot; end def handle_event(&quot;update_temp&quot;, %{&quot;code&quot; =&gt; &quot;ArrowUp&quot;}, socket) do {:ok, new_temp} = Thermostat.inc_temperature(socket.assigns.id) {:noreply, assign(socket, :temperature, new_temp)} end def handle_event(&quot;update_temp&quot;, %{&quot;code&quot; =&gt; &quot;ArrowDown&quot;}, socket) do {:ok, new_temp} = Thermostat.dec_temperature(socket.assigns.id) {:noreply, assign(socket, :temperature, new_temp)} end def handle_event(&quot;update_temp&quot;, _key, socket) do {:noreply, socket} end Compartmentalizing markup and events with render, live_render, and live_component We can render another template directly from a LiveView template by simply calling render: render &quot;child_template&quot;, assigns render SomeOtherView, &quot;child_template&quot;, assigns If the other template has the .leex extension, LiveView change tracking will also work across templates. When rendering a child template, any of the events bound in the child template will be sent to the parent LiveView. In other words, similar to regular Phoenix templates, a regular render call does not start another LiveView. This means render is useful to sharing markup between views. One option to address this problem is to render a child LiveView inside a parent LiveView by calling live_render/3 instead of render/3 from the LiveView template. This child LiveView runs in a completely separate process than the parent, with its own mount and handle_event callbacks. If a child LiveView crashes, it won&#39;t affect the parent. If the parent crashes, all children are terminated. When rendering a child LiveView, the :id option is required to uniquely identify the child. A child LiveView will only ever be rendered and mounted a single time, provided its ID remains unchanged. Updates to a child session will be merged on the client, but not passed back up until either a crash and re-mount or a connection drop and recovery. To force a child to re-mount with new session data, a new ID must be provided. Given a LiveView runs on its own process, it is an excellent tool for creating completely isolated UI elements, but it is a slightly expensive abstraction if all you want is to compartmentalize markup and events. For example, if you are showing a table with all users in the system, and you want to compartmentalize this logic, using a separate LiveView, each with its own process, would likely be too expensive. For these cases, LiveView provides Phoenix.LiveComponent, which are rendered using live_component/3: &lt;%= live_component(@socket, UserComponent, id: user.id, user: user) %&gt; Components have their own mount and handle_event callbacks, as well as their own state with change tracking support. Components are also lightweight as they &quot;run&quot; in the same process as the parent LiveView. However, this means an error in a component would cause the whole view to fail to render. See Phoenix.LiveComponent for a complete rundown on components. To sum it up: render - compartmentalizes markup live_component - compartmentalizes state, markup, and events live_render - compartmentalizes state, markup, events, and error isolation Live Layouts Your LiveView will be rendered within the layout specified in your Plug pipeline, such as the default app layout. Assigns defined during mount of the root LiveView are accessible in the layout, but the app layout is never updated after the initial render. For a live layout, you must specify an additional layout to use with your LiveView. For example, your regular app.html template may display a @new_message_count notification, like this: &lt;!DOCTYPE html&gt; &lt;html lang=&quot;en&quot;&gt; &lt;head&gt; &lt;title&gt;&lt;%= @page_title %&gt;&lt;/title&gt; &lt;/head&gt; &lt;body&gt; &lt;div&gt; &lt;nav&gt; ... Messages (&lt;%= @new_message_count %&gt;) &lt;/nav&gt; &lt;%= render @view_module, @view_template, assigns %&gt; &lt;/div&gt; &lt;/body&gt; &lt;/html&gt; To allow the @new_message_count to be be updated by your LiveView, you can move the dynamic content inside a sub-layout, such as app_web/templates/layout/live.html.leex. First, you would update your app.html layout to keep only the barebones HTML structure: &lt;!DOCTYPE html&gt; &lt;html lang=&quot;en&quot;&gt; &lt;head&gt; &lt;title&gt;...&lt;/title&gt; &lt;script&gt;...&lt;/script&gt; &lt;/head&gt; &lt;body&gt; &lt;%= render @view_module, @view_template, assigns %&gt; &lt;/body&gt; &lt;/html&gt; Next, define a new live.html.leex layout with the dynamic content, followed by a render of the inner @live_view_module: &lt;nav&gt; ... Messages (&lt;%= @new_message_count %&gt;) &lt;/nav&gt; &lt;%= @live_view_module.render(assigns) %&gt; Finally, update your LiveView to pass the :layout option to use Phoenix.LiveView: use Phoenix.LiveView, layout: {AppWeb.LayoutView, &quot;live.html&quot;} Or alternatively, you can provide the :layout dynamically as an option in mount: def mount(_session, socket) do socket = assign(socket, new_message_count: 0) {:ok, socket, layout: {AppWeb.LayoutView, &quot;live.html&quot;}} end def handle_info({:new_messages, count}, socket) do {:noreply, assign(socket, new_message_count: count)} end end Note: The layout will be wrapped by the LiveView&#39;s :container tag. Updating the HTML document title Because the main layout from the Plug pipeline is rendered outside of LiveView, the contents cannot be dynamically changed. The one exception is the &lt;title&gt; of the HTML document. Phoenix LiveView special cases the @page_title assign to allow dynamically updating the title of the page, which is useful when using live navigation, or annotating the browser tab with a notification. For example, to update the user&#39;s notification count in the browser&#39;s title bar, first set the page_title assign on mount: def mount(_session, socket) do socket = assign(socket, page_title: &quot;Latest Posts&quot;) {:ok, socket} end Then access @page_title in the app layout: &lt;title&gt;&lt;%= @page_title %&gt;&lt;/title&gt; Now, although the app layout is not updated by LiveView, by simply assigning to page_title, LiveView knows you want the title to be updated: def handle_info({:new_messages, count}, socket) do {:noreply, assign(socket, page_title: &quot;Latest Posts (\#{count} new)&quot;)} end Note: If you find yourself needing to dynamically patch other parts of the base layout, such as injecting new scripts or styles into the &lt;head&gt; during live navigation, a true page navigation should be used instead. Assigning the @page_title updates the document.title directly, and therefore cannot be used to update any other part of the base layout, even if the base layout references the assign. Rate limiting events with Debounce and Throttle All events can be rate-limited on the client by using the phx-debounce and phx-throttle bindings, with the following behavior: phx-debounce - Accepts either a string integer timeout value, or &quot;blur&quot;. When an int is provided, delays emitting the event by provided milliseconds. When &quot;blur&quot; is provided, delays emitting an input&#39;s change event until the field is blurred by the user. phx-throttle - Accepts an integer timeout value to throttle the event in milliseconds. Unlike debounce, throttle will immediately emit the event, then rate limit the event at one event per provided timeout. For example, to avoid validating an email until the field is blurred, while validating the username at most every 2 seconds after a user changes the field: &lt;form phx-change=&quot;validate&quot; phx-submit=&quot;save&quot;&gt; &lt;input type=&quot;text&quot; name=&quot;user[email]&quot; phx-debounce=&quot;blur&quot;/&gt; &lt;input type=&quot;text&quot; name=&quot;user[username]&quot; phx-debounce=&quot;2000&quot;/&gt; &lt;/form&gt; And to rate limit a button click to once every second: &lt;button phx-click=&quot;search&quot; phx-throttle=&quot;1000&quot;&gt;Search&lt;/button&gt; Likewise, you may throttle held-down keydown: &lt;div phx-window-keydown=&quot;keydown&quot; phx-throttle=&quot;500&quot;&gt; ... &lt;/div&gt; Unless held-down keys are required, a better approach is generally to use phx-keyup bindings which only trigger on key up, thereby being self-limiting. However, phx-keydown is useful for games and other usecases where a constant press on a key is desired. In such cases, throttle should always be used. Debounce and Throttle special behavior The following specialized behavior is performed for forms and keydown bindings: When a phx-submit, or a phx-change for a different input is triggered, any current debounce or throttle timers are reset for existing inputs. A phx-keydown binding is only throttled for key repeats. Unique keypresses back-to-back will dispatch the pressed key events. DOM patching and temporary assigns A container can be marked with phx-update, allowing the DOM patch operations to avoid updating or removing portions of the LiveView, or to append or prepend the updates rather than replacing the existing contents. This is useful for client-side interop with existing libraries that do their own DOM operations. The following phx-update values are supported: replace - the default operation. Replaces the element with the contents ignore - ignores updates to the DOM regardless of new content changes append - append the new DOM contents instead of replacing prepend - prepend the new DOM contents instead of replacing When using phx-update, a unique DOM ID must always be set in the container. If using &quot;append&quot; or &quot;prepend&quot;, a DOM ID must also be set for each child. When appending or prepending elements containing an ID already present in the container, LiveView will replace the existing element with the new content instead appending or prepending a new element. The &quot;ignore&quot; behaviour is frequently used when you need to integrate with another JS library. The &quot;append&quot; and &quot;prepend&quot; feature is often used with &quot;Temporary assigns&quot; to work with large amounts of data. Let&#39;s learn more. Temporary assigns By default, all LiveView assigns are stateful, which enables change tracking and stateful interactions. In some cases, it&#39;s useful to mark assigns as temporary, meaning they will be reset to a default value after each update, allowing otherwise large, but infrequently updated values to be discarded after the client has been patched. Imagine you want to implement a chat application with LiveView. You could render each message like this: &lt;%= for message &lt;- @messages do %&gt; &lt;p&gt;&lt;span&gt;&lt;%= message.username %&gt;:&lt;/span&gt; &lt;%= message.text %&gt;&lt;/p&gt; &lt;% end %&gt; Every time there is a new message, you would append it to the @messages assign and re-render all messages. As you may suspect, keeping the whole chat conversation in memory and resending it on every update would be too expensive, even with LiveView smart change tracking. By using temporary assigns and phx-update, we don&#39;t need to keep any message in memory and send messages to be appended to the UI only when there are new messages. To do so, the first step is to mark which assigns are temporary and what are the value they should be reset to on mount: def mount(_session, socket) do socket = assign(socket, :messages, load_last_20_messages()) {:ok, socket, temporary_assigns: [messages: []]} end On mount we also load the initial amount of messages we want to send. After the initial render, the initial batch of messages will be reset back to an empty list. Now, whenever there are one or more new messages, we will assign only the new messages to @messages: socket = assign(socket, :messages, new_messages) In the template, we want to wrap all of the messages in a container and tag this content with phx-update. Remember must also add an ID to the container as well as to each child: &lt;div id=&quot;chat-messages&quot; phx-update=&quot;append&quot;&gt; &lt;%= for message &lt;- @messages do %&gt; &lt;p id=&quot;&lt;%= message.id %&gt;&quot;&gt; &lt;span&gt;&lt;%= message.username %&gt;:&lt;/span&gt; &lt;%= message.text %&gt; &lt;/p&gt; &lt;% end %&gt; &lt;/div&gt; And now, once the client receives new messages, it knows it shouldn&#39;t replace the old content, but rather append to it. Live navigation The live_link/2 and live_redirect/2 functions allow page navigation using the browser&#39;s pushState API. With live navigation, the page is updated without a full page reload. To use live navigation, simply replace your existing Phoenix.HTML.link/3 and Phoenix.LiveView.redirect/2 calls with their live counterparts. For example, in a template you may write: &lt;%= live_link &quot;next&quot;, to: Routes.live_path(@socket, MyLive, @page + 1) %&gt; or in a LiveView: {:noreply, live_redirect(socket, to: Routes.live_path(socket, MyLive, page + 1))} When a live link is clicked, the following control flow occurs: if the route belongs to the existing root LiveView and the LiveView is defined in your application&#39;s router, the handle_params/3 callback is invoked without mounting a new LiveView. See the next section. if the route belongs to a different LiveView than the currently running root, then the existing root LiveView is shutdown, and an Ajax request is made to request the necessary information about the new LiveView, without performing a full static render (which reduces latency and improves performance). Once information is retrieved, the new LiveView is mounted. live_link/3 and live_redirect/2 are by default only available in LiveViews defined at the router with the live/3 macro. handle_params/3 The handle_params/3 callback is invoked after mount/2. It receives the request path parameters and the query parameters as first argument, the url as second, and the socket as third. As any other handle_* callback, changes to the state inside handle_params/3 will trigger a server render. To avoid building a new LiveView whenever a live link is clicked or whenever a live redirect happens, LiveView also invokes handle_params/3 on an existing LiveView when performing live navigation as long as: you are navigating to the same root live view you are currently on said LiveView is defined in your router For example, imagine you have a UserTable LiveView to show all users in the system and you define it in the router as: live &quot;/users&quot;, UserTable Now to add live sorting, you could do: &lt;%= live_link &quot;Sort by name&quot;, to: Routes.live_path(@socket, UserTable, %{sort_by: &quot;name&quot;}) %&gt; When clicked, since we are navigating to the current LiveView, handle_params/3 will be invoked. Remember you should never trust received params, so we can use the callback to validate the user input and change the state accordingly: def handle_params(params, _uri, socket) do case params[&quot;sort_by&quot;] do sort_by when sort_by in ~w(name company) -&gt; {:noreply, socket |&gt; assign(:sort_by, sort) |&gt; recompute_users()} _ -&gt; {:noreply, socket} end end Replace page address LiveView also allows the current browser URL to be replaced. This is useful when you want certain events to change the URL but without polluting the browser&#39;s history. For example, imagine there is a form that changes some page state when submitted. If those changes are not persisted in a database or similar, as soon as the user refreshes the page, navigates away, or shares the URL with someone else, said changes will be lost. To address this, users can invoke live_redirect/2. The idea is, once the form data is received, we do not change the state, instead we perform a live redirect to ourselves with the new URL. Since we are navigating to ourselves, handle_params/3 will be called with the new parameters, which we can then use to compute state and re-render the page. For example, let&#39;s change the &quot;sort by&quot; example from the previous page to perform sorting through a form. In other words, instead of sorting by clicking a &quot;Sort by name&quot; button, we will have a form with 2 radio buttons, that allows you to choose between sorting by name or company. Once the form is submitted, we can compute the new URL: def handle_event(&quot;sorting&quot;, params, socket) do {:noreply, live_redirect(socket, to: Routes.live_path(socket, __MODULE__, params))} end Now with a handle_params/3 implementation similar to the one in the previous section, we will recompute the users based on the new params and perform a server render if there are any changes. Both live_link/2 and live_redirect/2 support the replace: true option. This option can be used when you want to change the current url without polluting the browser&#39;s history: def handle_event(&quot;sorting&quot;, params, socket) do {:noreply, live_redirect(socket, to: Routes.live_path(socket, __MODULE__, params), replace: true)} end Disconnecting all instances of a given live user It is possible to identify all LiveView sockets by setting a &quot;live_socket_id&quot; in the session. For example, when signing in a user, you could do: conn |&gt; put_session(:current_user_id, user.id) |&gt; put_session(:live_socket_id, &quot;users_sockets:\#{user.id}&quot;) Now all LiveView sockets will be identified and listening to the given live_socket_id. You can disconnect all live users identified by said ID by broadcasting on the topic: MyApp.Endpoint.broadcast(&quot;users_socket:\#{user.id}&quot;, &quot;disconnect&quot;, %{}) It is the same mechanism provided by Phoenix.Socket, so you can use the same approach to disconnect live users and regular channels. JavaScript Client Specific As seen earlier, you start by instantiating a single LiveSocket instance to enable LiveView client/server interaction, for example: import {Socket} from &quot;phoenix&quot; import LiveSocket from &quot;phoenix_live_view&quot; let liveSocket = new LiveSocket(&quot;/live&quot;, Socket) liveSocket.connect() All options are passed directly to the Phoenix.Socket constructor, except for the following LiveView specific options: bindingPrefix - the prefix to use for phoenix bindings. Defaults &quot;phx-&quot; params - the connect_params to pass to the view&#39;s mount callback. May be a literal object or closure returning an object. When a closure is provided, the function receives the view&#39;s phx-view name. hooks – a reference to a user-defined hooks namespace, containing client callbacks for server/client interop. See the interop section below for details. Forms and input handling The JavaScript client is always the source of truth for current input values. For any given input with focus, LiveView will never overwrite the input&#39;s current value, even if it deviates from the server&#39;s rendered updates. This works well for updates where major side effects are not expected, such as form validation errors, or additive UX around the user&#39;s input values as they fill out a form. For these use cases, the phx-change input does not concern itself with disabling input editing while an event to the server is inflight. When a phx-change event is sent to the server, a &quot;_target&quot; param will be in the root payload containing the keyspace of the input name which triggered the change event. For example, if the following input triggered a change event: &lt;input name=&quot;user[username]&quot;/&gt; The server&#39;s handle_event/3 would receive a payload: %{&quot;_target&quot; =&gt; [&quot;user&quot;, &quot;username&quot;], &quot;user&quot; =&gt; %{&quot;name&quot; =&gt; &quot;Name&quot;}} The phx-submit event is used for form submissions where major side-effects typically happen, such as rendering new containers, calling an external service, or redirecting to a new page. For these use-cases, the form inputs are set to readonly on submit, and any submit button is disabled until the client gets an acknowledgment that the server has processed the phx-submit event. Following an acknowledgment, any updates are patched to the DOM as normal, and the last input with focus is restored if the user has not otherwise focused on a new input during submission. To handle latent form submissions, any HTML tag can be annotated with phx-disable-with, which swaps the element&#39;s innerText with the provided value during form submission. For example, the following code would change the &quot;Save&quot; button to &quot;Saving...&quot;, and restore it to &quot;Save&quot; on acknowledgment: &lt;button type=&quot;submit&quot; phx-disable-with=&quot;Saving...&quot;&gt;Save&lt;/button&gt; Loading state and errors By default, the following classes are applied to the LiveView&#39;s parent container: &quot;phx-connected&quot; - applied when the view has connected to the server &quot;phx-disconnected&quot; - applied when the view is not connected to the server &quot;phx-error&quot; - applied when an error occurs on the server. Note, this class will be applied in conjunction with &quot;phx-disconnected&quot; if connection to the server is lost. When a form bound with phx-submit is submitted, the &quot;phx-loading&quot; class is applied to the form, which is removed on update. JS Interop and client controlled DOM To handle custom client-side javascript when an element is added, updated, or removed by the server, a hook object may be provided with the following life-cycle callbacks: mounted - the element has been added to the DOM and its server LiveView has finished mounting beforeUpdate - the element is about to be updated in the DOM. Note: any call here must be synchronous as the operation cannot be deferred or cancelled. updated - the element has been updated in the DOM by the server beforeDestroy - the element is about to be removed from the DOM. Note: any call here must be synchronous as the operation cannot be deferred or cancelled. destroyed - the element has been removed from the page, either by a parent update, or the parent being removed entirely disconnected - the element&#39;s parent LiveView has disconnected from the server reconnected - the element&#39;s parent LiveView has reconnected to the server In addition to the callbacks, the callbacks contain the following attributes in scope: el - attribute referencing the bound DOM node, viewName - attribute matching the dom node&#39;s phx-view value pushEvent(event, payload) - method to push an event from the client to the LiveView server pushEventTo(selector, event, payload) - method to push targeted events from the client to LiveViews and LiveComponents. For example, a controlled input for phone-number formatting would annotate their markup: &lt;input type=&quot;text&quot; name=&quot;user[phone_number]&quot; id=&quot;user-phone-number&quot; phx-hook=&quot;PhoneNumber&quot; /&gt; Then a hook callback object can be defined and passed to the socket: let Hooks = {} Hooks.PhoneNumber = { mounted() { this.el.addEventListener(&quot;input&quot;, e =&gt; { let match = this.el.value.replace(/\\\\D/g, &quot;&quot;).match(/^(\\\\d{3})(\\\\d{3})(\\\\d{4})$/) if(match) { this.el.value = `${match[1]}-${match[2]}-${match[3]}` } }) } } let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, {hooks: Hooks}) ... Note: when using phx-hook, a unique DOM ID must always be set. Endpoint configuration LiveView accepts the following configuration in your endpoint under the :live_view key: :signing_salt (required) - the salt used to sign data sent to the client :hibernate_after (optional) - the amount of time in miliseconds of inactivity inside the LiveView so it hibernates (i.e. it compresses its own memory and state). Defaults to 15000ms (15 seconds)"},{"ref":"Phoenix.LiveView.html#__using__/1","title":"Phoenix.LiveView.__using__/1","type":"macro","doc":"Uses LiveView in the current module to mark it a LiveView. use Phoenix.LiveView, namespace: MyAppWeb, container: {:tr, class: &quot;colorized&quot;}, layout: {MyAppWeb.LayoutView, &quot;live.html&quot;} Options :namespace - configures the namespace the LiveView is in :container - configures the container the LiveView will be wrapped in :layout - configures the layout the LiveView will be rendered in"},{"ref":"Phoenix.LiveView.html#assign/2","title":"Phoenix.LiveView.assign/2","type":"function","doc":"See assign/3."},{"ref":"Phoenix.LiveView.html#assign/3","title":"Phoenix.LiveView.assign/3","type":"function","doc":"Adds key value pairs to socket assigns. A single key value pair may be passed, or a keyword list of assigns may be provided to be merged into existing socket assigns. Examples iex&gt; assign(socket, :name, &quot;Elixir&quot;) iex&gt; assign(socket, name: &quot;Elixir&quot;, logo: &quot;💧&quot;)"},{"ref":"Phoenix.LiveView.html#assign_new/3","title":"Phoenix.LiveView.assign_new/3","type":"function","doc":"Assigns a value into the socket only if it does not exist. Useful for lazily assigning values and referencing parent assigns. Referencing parent assigns When a LiveView is mounted in a disconnected state, the Plug.Conn assigns will be available for reference via assign_new/3, allowing assigns to be shared for the initial HTTP request. On connected mount, the assign_new/3 would be invoked, and the LiveView would use its session to rebuild the originally shared assign. Likewise, nested LiveView children have access to their parent&#39;s assigns on mount using assign_new, which allows assigns to be shared down the nested LiveView tree. Examples # controller conn |&gt; assign(:current_user, user) |&gt; LiveView.Controller.live_render(MyLive, session: %{&quot;user_id&quot; =&gt; user.id}) # LiveView mount def mount(%{&quot;user_id&quot; =&gt; user_id}, socket) do {:ok, assign_new(socket, :current_user, fn -&gt; Accounts.get_user!(user_id) end)} end"},{"ref":"Phoenix.LiveView.html#connected?/1","title":"Phoenix.LiveView.connected?/1","type":"function","doc":"Returns true if the socket is connected. Useful for checking the connectivity status when mounting the view. For example, on initial page render, the view is mounted statically, rendered, and the HTML is sent to the client. Once the client connects to the server, a LiveView is then spawned and mounted statefully within a process. Use connected?/1 to conditionally perform stateful work, such as subscribing to pubsub topics, sending messages, etc. Examples defmodule DemoWeb.ClockLive do use Phoenix.LiveView ... def mount(_session, socket) do if connected?(socket), do: :timer.send_interval(1000, self(), :tick) {:ok, assign(socket, date: :calendar.local_time())} end def handle_info(:tick, socket) do {:noreply, assign(socket, date: :calendar.local_time())} end end"},{"ref":"Phoenix.LiveView.html#get_connect_params/1","title":"Phoenix.LiveView.get_connect_params/1","type":"function","doc":"Accesses the connect params sent by the client for use on connected mount. Connect params are only sent when the client connects to the server and only remain available during mount. nil is returned when called in a disconnected state and a RuntimeError is raised if called after mount. Examples def mount(_session, socket) do {:ok, assign(socket, width: get_connect_params(socket)[&quot;width&quot;] || @width)} end"},{"ref":"Phoenix.LiveView.html#c:handle_call/3","title":"Phoenix.LiveView.handle_call/3","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#c:handle_event/3","title":"Phoenix.LiveView.handle_event/3","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#c:handle_info/2","title":"Phoenix.LiveView.handle_info/2","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#c:handle_params/3","title":"Phoenix.LiveView.handle_params/3","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#live_redirect/2","title":"Phoenix.LiveView.live_redirect/2","type":"function","doc":"Annotates the socket for navigation without a page refresh. When navigating to a path which routes to your existing LiveView, the handle_params/3 callback is immediately invoked in your existing LiveView process to handle the change of URL state. For live redirects to external LiveViews, the existing LiveView is shutdown. Options :to - the required path to link to. It must always be a local path :replace - the flag to replace the current history or push a new state. Defaults false. Examples {:noreply, live_redirect(socket, to: &quot;/&quot;)} {:noreply, live_redirect(socket, to: &quot;/&quot;, replace: true)}"},{"ref":"Phoenix.LiveView.html#c:mount/2","title":"Phoenix.LiveView.mount/2","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#put_flash/3","title":"Phoenix.LiveView.put_flash/3","type":"function","doc":"Adds a flash message to the socket to be displayed on redirect. Note: the Phoenix.LiveView.Flash plug must be plugged in your browser&#39;s pipeline for flash to be supported, for example: pipeline :browser do plug :accepts, [&quot;html&quot;] plug :fetch_session plug Phoenix.LiveView.Flash ... end Examples iex&gt; put_flash(socket, :info, &quot;It worked!&quot;) iex&gt; put_flash(socket, :error, &quot;You can&#39;t access that page&quot;)"},{"ref":"Phoenix.LiveView.html#redirect/2","title":"Phoenix.LiveView.redirect/2","type":"function","doc":"Annotates the socket for redirect to a destination path. Note: LiveView redirects rely on instructing client to perform a window.location update on the provided redirect location. Options :to - the path to redirect to. It must always be a local path :external - an external path to redirect to"},{"ref":"Phoenix.LiveView.html#c:render/1","title":"Phoenix.LiveView.render/1","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#send_update/2","title":"Phoenix.LiveView.send_update/2","type":"function","doc":"Asynchronously updates a component with new assigns. Requires a stateful component with a matching :id to send the update to. Following the optional preload/1 callback being invoked, the updated values are merged with the component&#39;s assigns and update/2 is called for the updated component(s). While a component may always be updated from the parent by updating some parent assigns which will re-render the child, thus invoking update/2 on the child component, send_update/2 is useful for updating a component that entirely manages its own state, as well as messaging between components. Examples def handle_event(&quot;cancel-order&quot;, _, socket) do ... send_update(Cart, id: &quot;cart&quot;, status: &quot;cancelled&quot;) {:noreply, socket} end"},{"ref":"Phoenix.LiveView.html#c:terminate/2","title":"Phoenix.LiveView.terminate/2","type":"callback","doc":""},{"ref":"Phoenix.LiveView.html#update/3","title":"Phoenix.LiveView.update/3","type":"function","doc":"Updates an existing key in the socket assigns. The update function receives the current key&#39;s value and returns the updated value. Raises if the key does not exist. Examples iex&gt; update(socket, :count, fn count -&gt; count + 1 end) iex&gt; update(socket, :count, &amp;(&amp;1 + 1))"},{"ref":"Phoenix.LiveView.Controller.html","title":"Phoenix.LiveView.Controller","type":"module","doc":"The Controller for LiveView rendering."},{"ref":"Phoenix.LiveView.Controller.html#live_render/3","title":"Phoenix.LiveView.Controller.live_render/3","type":"function","doc":"Renders a live view from a Plug request and sends an HTML response. Before render the @live_view_module assign will be added to the connection assigns for reference. Options See Phoenix.LiveView.live_render/3 for all supported options. Examples defmodule ThermostatController do ... import Phoenix.LiveView.Controller def show(conn, %{&quot;id&quot; =&gt; thermostat_id}) do live_render(conn, ThermostatLive, session: %{ &quot;thermostat_id&quot; =&gt; id, &quot;current_user_id&quot; =&gt; get_session(conn, :user_id) }) end end"},{"ref":"Phoenix.LiveView.Flash.html","title":"Phoenix.LiveView.Flash","type":"module","doc":"Fetches Phoenix LiveView flash messages from cookie token. Examples plug Phoenix.LiveView.Flash By default, the signing salt for the token is pulled from your endpoint&#39;s LiveView config, for example: config :my_app, MyAppWeb.Endpoint, ..., live_view: [signing_salt: ...] The :signing_salt option may also be passed directly to the plug."},{"ref":"Phoenix.LiveView.Flash.html#call/2","title":"Phoenix.LiveView.Flash.call/2","type":"function","doc":"Fetches the live flash from a token passed via client cookies."},{"ref":"Phoenix.LiveView.Helpers.html","title":"Phoenix.LiveView.Helpers","type":"module","doc":"A collection of helpers to be imported into your views."},{"ref":"Phoenix.LiveView.Helpers.html#live_component/4","title":"Phoenix.LiveView.Helpers.live_component/4","type":"macro","doc":"Renders a Phoenix.LiveComponent within a parent LiveView. While LiveViews can be nested, each LiveView starts its own process. A LiveComponent provides similar functionality to LiveView, except they run in the same process as the LiveView, with its own encapsulated state. LiveComponent comes in two shapes, stateful and stateless. See Phoenix.LiveComponent for more information. Examples All of the assigns given are forwarded directly to the live_component: &lt;%= live_component(@socket, MyApp.WeatherComponent, id: &quot;thermostat&quot;, city: &quot;Kraków&quot;) %&gt; Note the :id won&#39;t necessarily be used as the DOM ID. That&#39;s up to the component. However, note the :id has a special meaning: whenever an :id is given, the component becomes stateful. Otherwise, :id is always set to nil."},{"ref":"Phoenix.LiveView.Helpers.html#live_link/1","title":"Phoenix.LiveView.Helpers.live_link/1","type":"function","doc":"Generates a live link for HTML5 pushState based navigation without page reloads. Options :to - the required path to link to. :replace - the flag to replace the current history or push a new state. Defaults false. All other options are forwarded to the anchor tag. Examples &lt;%= live_link &quot;next&quot;, to: Routes.live_path(@socket, MyLive, @page + 1) %&gt; &lt;%= live_link to: Routes.live_path(@socket, MyLive, dir: :asc), replace: false do %&gt; Sort By Price &lt;% end %&gt;"},{"ref":"Phoenix.LiveView.Helpers.html#live_link/2","title":"Phoenix.LiveView.Helpers.live_link/2","type":"function","doc":""},{"ref":"Phoenix.LiveView.Helpers.html#live_render/3","title":"Phoenix.LiveView.Helpers.live_render/3","type":"function","doc":"Renders a LiveView within an originating plug request or within a parent LiveView. Options :session - the map of extra session data to be serialized and sent to the client. Note all session data currently in the connection is automatically available in LiveViews. You can use this option to provide extra data. Also note the keys in the session are strings keys, as a reminder that data has to be serialized first. :container - the optional tuple for the HTML tag and DOM attributes to be used for the LiveView container. For example: {:li, style: &quot;color: blue;&quot;}. By default it uses the module definition container. See the &quot;Containers&quot; section for more information. :id - both the DOM ID and the ID to uniquely identify a LiveView. One :id is automatically generated when rendering root LiveViews but it is a required option when rendering a child LiveView. :router - an optional router that enables this LiveView to perform live_link and live_redirect. Only a single LiveView in a page may have the :router set and it will effectively become the view responsible for handling live_link and live_redirect. LiveViews defined at the router with the live macro automatically have the :router option set. Examples # within eex template &lt;%= live_render(@conn, MyApp.ThermostatLive) %&gt; # within leex template &lt;%= live_render(@socket, MyApp.ThermostatLive, id: &quot;thermostat&quot;) %&gt; Containers When a LiveView is rendered, its contents are wrapped in a container. By default, said container is a div tag with a handful of LiveView specific attributes. The container can be customized in different ways: You can change the default container on use Phoenix.LiveView: use Phoenix.LiveView, container: {:tr, id: &quot;foo-bar&quot;} You can override the container tag and pass extra attributes when calling live_render (as well as on your live call in your router): live_render socket, MyLiveView, container: {:tr, class: &quot;highlight&quot;}"},{"ref":"Phoenix.LiveView.Helpers.html#sigil_L/2","title":"Phoenix.LiveView.Helpers.sigil_L/2","type":"macro","doc":"Provides ~L sigil with HTML safe Live EEx syntax inside source files. iex&gt; ~L&quot;&quot;&quot; ...&gt; Hello &lt;%= &quot;world&quot; %&gt; ...&gt; &quot;&quot;&quot; {:safe, [&quot;Hello &quot;, &quot;world&quot;, &quot;\\n&quot;]}"},{"ref":"Phoenix.LiveView.Router.html","title":"Phoenix.LiveView.Router","type":"module","doc":"Provides LiveView routing for Phoenix routers."},{"ref":"Phoenix.LiveView.Router.html#live/3","title":"Phoenix.LiveView.Router.live/3","type":"macro","doc":"Defines a LiveView route. Layout When a layout isn&#39;t explicitly set, a default layout is inferred similar to controller actions. For example, the layout for the router MyAppWeb.Router would be inferred as MyAppWeb.LayoutView and would use the :app template. Options :session - the optional list of keys to pull out of the Plug connection session and into the LiveView session. For example, the following would copy Plug&#39;s session current user ID and the remember_me value into the LiveView session: [:user_id, :remember_me] This also accepts key/value entries which are mapped directly into the LiveView session. This is useful for identifying which live macro was matched. [action: :index] :layout - the optional tuple for specifying a layout to render the LiveView. Defaults to {LayoutView, :app} where LayoutView is relative to your application&#39;s namespace. :container - the optional tuple for the HTML tag and DOM attributes to be used for the LiveView container. For example: {:li, style: &quot;color: blue;&quot;}. See Phoenix.LiveView.live_render/3 for more information on examples. :as - optionally configures the named helper. Defaults to :live. Examples defmodule MyApp.Router use Phoenix.Router import Phoenix.LiveView.Router scope &quot;/&quot;, MyApp do pipe_through [:browser] live &quot;/thermostat&quot;, ThermostatLive live &quot;/clock&quot;, ClockLive, session: [:user_id] live &quot;/dashboard&quot;, DashboardLive, layout: {MyApp.AlternativeView, &quot;app.html&quot;} end end iex&gt; MyApp.Router.Helpers.live_path(MyApp.Endpoint, MyApp.ThermostatLive) &quot;/thermostat&quot;"},{"ref":"Phoenix.LiveView.Socket.html","title":"Phoenix.LiveView.Socket","type":"module","doc":"The LiveView socket for Phoenix Endpoints."},{"ref":"Phoenix.LiveView.Socket.html#connect/3","title":"Phoenix.LiveView.Socket.connect/3","type":"function","doc":"Connects the Phoenix.Socket for a LiveView client."},{"ref":"Phoenix.LiveView.Socket.html#id/1","title":"Phoenix.LiveView.Socket.id/1","type":"function","doc":"Identifies the Phoenix.Socket for a LiveView client."},{"ref":"Phoenix.LiveView.Socket.html#t:assigns/0","title":"Phoenix.LiveView.Socket.assigns/0","type":"type","doc":""},{"ref":"Phoenix.LiveView.Socket.html#t:t/0","title":"Phoenix.LiveView.Socket.t/0","type":"type","doc":""},{"ref":"Phoenix.LiveView.Socket.html#t:unsigned_params/0","title":"Phoenix.LiveView.Socket.unsigned_params/0","type":"type","doc":""},{"ref":"Phoenix.LiveViewTest.html","title":"Phoenix.LiveViewTest","type":"module","doc":"Conveniences for testing Phoenix live views. In LiveView tests, we interact with views via process communication in substitution of a browser. Like a browser, our test process receives messages about the rendered updates from the view which can be asserted against to test the life-cycle and behavior of live views and their children. LiveView Testing The life-cycle of a live view as outlined in the Phoenix.LiveView docs details how a view starts as a stateless HTML render in a disconnected socket state. Once the browser receives the HTML, it connects to the server and a new LiveView process is started, remounted in a connected socket state, and the view continues statefully. The LiveView test functions support testing both disconnected and connected mounts separately, for example: use Phoenix.ConnTest import Phoenix.LiveViewTest @endpoint MyEndpoint test &quot;disconnected and connected mount&quot;, %{conn: conn} do conn = get(conn, &quot;/my-path&quot;) assert html_response(conn, 200) =~ &quot;&lt;h1&gt;My Disconnected View&lt;/h1&gt;&quot; {:ok, view, html} = live(conn) end test &quot;redirected mount&quot;, %{conn: conn} do assert {:error, %{redirect: %{to: &quot;/somewhere&quot;}}} = live(conn, &quot;my-path&quot;) end Here, we start by using the familiar Phoenix.ConnTest function, get/2 to test the regular HTTP get request which invokes mount with a disconnect socket. Next, live/1 is called with our sent connection to mount the view in a connected state, which starts our stateful LiveView process. In general, it&#39;s often more convenient to test the mounting of a view in a single step, provided you don&#39;t need the result of the stateless HTTP render. This is done with a single call to live/2, which performs the get step for us: test &quot;connected mount&quot;, %{conn: conn} do {:ok, view, html} = live(conn, &quot;/my-path&quot;) assert html =~ &quot;&lt;h1&gt;My Connected View&lt;/h1&gt;&quot; end Testing Events The browser can send a variety of events to a live view via phx- bindings, which are sent to the handle_event/3 callback. To test events sent by the browser and assert on the rendered side-effect of the event, use the render_* functions: render_click/3 - sends a phx-click event and value and returns the rendered result of the handle_event/3 callback. render_submit/3 - sends a form phx-submit event and value and returns the rendered result of the handle_event/3 callback. render_change/3 - sends a form phx-change event and value and returns the rendered result of the handle_event/3 callback. render_keydown/3 - sends a form phx-keydown event and value and returns the rendered result of the handle_event/3 callback. render_keyup/3 - sends a form phx-keyup event and value and returns the rendered result of the handle_event/3 callback. For example: {:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert render_click(view, :inc) =~ &quot;The temperature is: 31℉&quot; assert render_click(view, :set_temp, 35) =~ &quot;The temperature is: 35℉&quot; assert render_submit(view, :save, %{deg: 30}) =~ &quot;The temperature is: 30℉&quot; assert render_change(view, :validate, %{deg: -30}) =~ &quot;invalid temperature&quot; assert render_keydown(view, :key, :ArrowUp) =~ &quot;The temperature is: 31℉&quot; assert render_keydown(view, :key, :ArrowDown) =~ &quot;The temperature is: 30℉&quot; Testing regular messages Live views are GenServer&#39;s under the hood, and can send and receive messages just like any other server. To test the side effects of sending or receiving messages, simply message the view and use the render function to test the result: send(view.pid, {:set_temp: 50}) assert render(view) =~ &quot;The temperature is: 50℉&quot; Testing shutdowns and stopping views Like all processes, views can shutdown normally or abnormally, and this can be tested with assert_remove/3. For example: send(view.pid, :boom) assert_remove view, {:shutdown, %RuntimeError{}} stop(view) assert_remove view, {:shutdown, :stop} Nested views can be removed by a parent at any time based on conditional rendering. In these cases, the removal of the view is detected by the browser, or our test client, and the child is shutdown gracefully. This can be tested in the same way as above: assert render(parent) =~ &quot;some content in child&quot; assert child = find_child(parent, &quot;child-dom-id&quot;) send(parent.pid, :msg_that_removes_child) assert_remove child, _ refute render(parent) =~ &quot;some content in child&quot; Testing components There are two main mechanisms for testing components. To test stateless components or just a regular rendering of a component, one can use render_component/2: assert render_component(MyComponent, id: 123, user: %User{}) =~ &quot;some markup in component&quot; If you want to test how components are mounted by a LiveView and interact with DOM events, you can use the regular live/2 macro to build the LiveView with the component and then scope events by passing the view and the component DOM ID in a list: {:ok, view, html} = live(conn, &quot;/users&quot;) html = render_click([view, &quot;user-13&quot;], &quot;delete&quot;, %{}) refute html =~ &quot;user-13&quot; assert_remove_component(view, &quot;user-13&quot;)"},{"ref":"Phoenix.LiveViewTest.html#assert_redirect/3","title":"Phoenix.LiveViewTest.assert_redirect/3","type":"macro","doc":"Asserts a redirect was peformed after execution of the provided function. Examples assert_redirect view, &quot;/path&quot;, fn -&gt; assert render_click(view, :event_that_triggers_redirect) end"},{"ref":"Phoenix.LiveViewTest.html#assert_remove/3","title":"Phoenix.LiveViewTest.assert_remove/3","type":"macro","doc":"Asserts a view was removed by a parent or shutdown itself. Examples [child1, child2] = children(parent_view) send(parent_view.pid, :msg_that_removes_child) assert_remove child1, _ assert_remove child2, {:shutdown, :removed}"},{"ref":"Phoenix.LiveViewTest.html#children/1","title":"Phoenix.LiveViewTest.children/1","type":"function","doc":"Returns the current list of children of the parent live view. Children are return in the order they appear in the rendered HTML. Examples {:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert [clock_view] = children(view) assert render_click(clock_view, :snooze) =~ &quot;snoozing&quot;"},{"ref":"Phoenix.LiveViewTest.html#find_child/2","title":"Phoenix.LiveViewTest.find_child/2","type":"function","doc":"Finds the nested LiveView child given a parent. Examples {:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert clock_view = find_child(view, &quot;clock&quot;) assert render_click(clock_view, :snooze) =~ &quot;snoozing&quot;"},{"ref":"Phoenix.LiveViewTest.html#live/2","title":"Phoenix.LiveViewTest.live/2","type":"macro","doc":"Spawns a connected LiveView process. Accepts either a previously rendered %Plug.Conn{} or an unsent %Plug.Conn{}. The latter case is a convenience to perform the get/2 and connected mount in a single step. Options :connect_params - the map of params available in the socket connected mount. See Phoenix.LiveView.get_connect_params/1 for more information. Examples {:ok, view, html} = live(conn, &quot;/path&quot;) assert view.module = MyLive assert html =~ &quot;the count is 3&quot; assert {:error, %{redirect: %{to: &quot;/somewhere&quot;}}} = live(conn, &quot;/path&quot;) {:ok, view, html} = conn |&gt; get(&quot;/path&quot;) |&gt; live()"},{"ref":"Phoenix.LiveViewTest.html#live/3","title":"Phoenix.LiveViewTest.live/3","type":"macro","doc":"See live/2."},{"ref":"Phoenix.LiveViewTest.html#live_isolated/3","title":"Phoenix.LiveViewTest.live_isolated/3","type":"macro","doc":"Spawns a connected LiveView process mounted in isolation as the sole rendered element. Useful for testing LiveViews that are not directly routable, such as those built as small components to be re-used in multiple parents. Testing routable LiveViews is still recommended whenever possible since features such as live navigation require routable LiveViews. Options :connect_params - the map of params available in connected mount. See Phoenix.LiveView.get_connect_params/1 for more information. :session - the session to be given to the LiveView All other options are forwarded to the live view for rendering. Refer to Phoenix.LiveView.live_render/3 for list of supported render options. Examples {:ok, view, html} = live_isolated(conn, AppWeb.ClockLive, session: %{&quot;tz&quot; =&gt; &quot;EST&quot;})"},{"ref":"Phoenix.LiveViewTest.html#render/1","title":"Phoenix.LiveViewTest.render/1","type":"function","doc":"Returns the string of HTML of the rendered view or component. If a view is provided, the entire LiveView is rendered. A list may be passed containing view and the component DOM ID&#39;s: Examples {:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert render(view) =~ &quot;cooling&quot; assert render([view, &quot;clock&quot;]) =~ &quot;12:00 PM&quot; assert render([view, &quot;clock&quot;, &quot;alarm&quot;]) =~ &quot;Snooze&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_blur/3","title":"Phoenix.LiveViewTest.render_blur/3","type":"function","doc":"Sends a blur event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_blur(view, :inactive) =~ &quot;Tap to wake&quot; assert render_blur([view, &quot;child-id&quot;], :inactive) =~ &quot;Tap to wake&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_change/3","title":"Phoenix.LiveViewTest.render_change/3","type":"function","doc":"Sends a form change event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_change(view, :validate, %{deg: 123}) =~ &quot;123 exceeds limits&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_click/3","title":"Phoenix.LiveViewTest.render_click/3","type":"function","doc":"Sends a click event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temperature is: 30℉&quot; assert render_click(view, :inc) =~ &quot;The temperature is: 31℉&quot; assert render_click([view, &quot;clock&quot;], :set, %{time: &quot;1:00&quot;}) =~ &quot;time: 1:00 PM&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_component/2","title":"Phoenix.LiveViewTest.render_component/2","type":"macro","doc":"Mounts, updates and renders a component. Examples assert render_component(MyComponent, id: 123, user: %User{}) =~ &quot;some markup in component&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_focus/3","title":"Phoenix.LiveViewTest.render_focus/3","type":"function","doc":"Sends a focus event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_blur(view, :inactive) =~ &quot;Tap to wake&quot; assert render_focus(view, :active) =~ &quot;Waking up...&quot; assert render_focus([view, &quot;child-id&quot;], :active) =~ &quot;Waking up...&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_hook/3","title":"Phoenix.LiveViewTest.render_hook/3","type":"function","doc":"Sends a hook event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_hook(view, :refresh, %{deg: 32}) =~ &quot;The temp is: 32℉&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_keydown/3","title":"Phoenix.LiveViewTest.render_keydown/3","type":"function","doc":"Sends a keydown event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_keyup(view, :inc, :ArrowUp) =~ &quot;The temp is: 32℉&quot; assert render_keyup([view, &quot;child-id&quot;], :inc, :ArrowDown) =~ &quot;The temp is: 31℉&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_keyup/3","title":"Phoenix.LiveViewTest.render_keyup/3","type":"function","doc":"Sends a keyup event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_keyup(view, :inc, :ArrowUp) =~ &quot;The temp is: 32℉&quot; assert render_keyup([view, &quot;child-id&quot;], :inc, :ArrowDown) =~ &quot;The temp is: 31℉&quot;"},{"ref":"Phoenix.LiveViewTest.html#render_live_link/2","title":"Phoenix.LiveViewTest.render_live_link/2","type":"function","doc":"Simulates a live_link click to the view and returns the rendered result."},{"ref":"Phoenix.LiveViewTest.html#render_submit/3","title":"Phoenix.LiveViewTest.render_submit/3","type":"function","doc":"Sends a form submit event to the view and returns the rendered result. Examples {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_submit(view, :refresh, %{deg: 32}) =~ &quot;The temp is: 32℉&quot;"},{"ref":"Phoenix.LiveViewTest.html#stop/1","title":"Phoenix.LiveViewTest.stop/1","type":"function","doc":"Stops a LiveView process. Examples stop(view) assert_remove view, {:shutdown, :stop}"},{"ref":"Phoenix.LiveViewTest.View.html","title":"Phoenix.LiveViewTest.View","type":"module","doc":"The struct for testing LiveViews. The following public fields represent the LiveView: id - The DOM id of the LiveView module - The module of the running LiveView pid - The Pid of the running LiveView See the Phoenix.LiveViewTest documentation for usage."},{"ref":"Phoenix.LiveView.Component.html","title":"Phoenix.LiveView.Component","type":"module","doc":"The struct returned by components in .leex templates. This component is never meant to be output directly into the template. It should always be handled by the diffing algorithm."},{"ref":"Phoenix.LiveView.Component.html#t:t/0","title":"Phoenix.LiveView.Component.t/0","type":"type","doc":""},{"ref":"Phoenix.LiveView.Comprehension.html","title":"Phoenix.LiveView.Comprehension","type":"module","doc":"The struct returned by for-comprehensions in .leex templates. See a description about its fields and use cases in Phoenix.LiveView.Engine docs."},{"ref":"Phoenix.LiveView.Comprehension.html#t:t/0","title":"Phoenix.LiveView.Comprehension.t/0","type":"type","doc":""},{"ref":"Phoenix.LiveView.Engine.html","title":"Phoenix.LiveView.Engine","type":"module","doc":"The .leex (Live EEx) template engine that tracks changes. On the docs below, we will explain how it works internally. For user-facing documentation, see Phoenix.LiveView. Phoenix.LiveView.Rendered Whenever you render a .leex template, it returns a Phoenix.LiveView.Rendered structure. This structure has three fields: :static, :dynamic and :fingerprint. The :static field is a list of literal strings. This allows the Elixir compiler to optimize this list and avoid allocating its strings on every render. The :dynamic field contains a list of dynamic content. Each element in the list is either one of: iodata - which is the dynamic content nil - the dynamic content did not change, see &quot;Tracking changes&quot; below another Phoenix.LiveView.Rendered struct, see &quot;Nesting and fingerprinting&quot; below a Phoenix.LiveView.Comprehension struct, see &quot;Comprehensions&quot; below a Phoenix.LiveView.Component struct, see &quot;Component&quot; below When you render a .leex template, you can convert the rendered structure to iodata by intercalating the static and dynamic fields, always starting with a static entry followed by a dynamic entry. The last entry will always be static too. So the following structure: %Phoenix.LiveView.Rendered{ static: [&quot;foo&quot;, &quot;bar&quot;, &quot;baz&quot;], dynamic: [&quot;left&quot;, &quot;right&quot;] } Results in the following content to be sent over the wire as iodata: [&quot;foo&quot;, &quot;left&quot;, &quot;bar&quot;, &quot;right&quot;, &quot;baz&quot;] This is also what calling Phoenix.HTML.Safe.to_iodata/1 with a Phoenix.LiveView.Rendered structure returns. Of course, the benefit of .leex templates is exactly that you do not need to send both static and dynamic segments every time. So let&#39;s talk about tracking changes. Tracking changes By default, a .leex template does not track changes. Change tracking can be enabled by passing a socket with two keys :fingerprints and :changed. If the :fingerprints matches the template fingerprint, then the :changed map is used. The map should contain the name of any changed field as key and the boolean true as value. If a field is not listed in :changed, then it is always considered unchanged. If a field is unchanged and .leex believes a dynamic expression no longer needs to be computed, its value in the dynamic list will be nil. This information can be leveraged to avoid sending data to the client. Nesting and fingerprinting Phoenix.LiveView also tracks changes across .leex templates. Therefore, if your view has this: &lt;%= render &quot;form.html&quot;, assigns %&gt; Phoenix will be able to track what is static and dynamic across templates, as well as what changed. A rendered nested .leex template will appear in the dynamic list as another Phoenix.LiveView.Rendered structure, which must be handled recursively. However, because the rendering of live templates can be dynamic in itself, it is important to distinguish which .leex template was rendered. For example, imagine this code: &lt;%= if something?, do: render(&quot;one.html&quot;, assigns), else: render(&quot;other.html&quot;, assigns) %&gt; To solve this, all Phoenix.LiveView.Rendered structs also contain a fingerprint field that uniquely identifies it. If the fingerprints are equal, you have the same template, and therefore it is possible to only transmit its changes. Comprehensions Another optimization done by .leex templates is to track comprehensions. If your code has this: &lt;%= for point &lt;- @points do %&gt; x: &lt;%= point.x %&gt; y: &lt;%= point.y %&gt; &lt;% end %&gt; Instead of rendering all points with both static and dynamic parts, it returns a Phoenix.LiveView.Comprehension struct with the static parts, that are shared across all points, and a list of dynamics to be interpolated inside the static parts. If @points is a list with %{x: 1, y: 2} and %{x: 3, y: 4}, the expression above would return: %Phoenix.LiveView.Comprehension{ static: [&quot;\\n x: &quot;, &quot;\\n y: &quot;, &quot;\\n&quot;], dynamics: [ [&quot;1&quot;, &quot;2&quot;], [&quot;3&quot;, &quot;4&quot;] ] } This allows .leex templates to drastically optimize the data sent by comprehensions, as the static parts are emitted once, regardless of the number of items. The list of dynamics is always a list of iodatas or components, as we don&#39;t perform change tracking inside the comprehensions themselves. Similarly, comprehensions do not have fingerprints because they are only optimized at the root, so conditional evaluation, as the one seen in rendering, is not possible. The only possible outcome for a dynamic field that returns a comprehension is nil. Components .leex also supports stateful components. Since they are stateful, they are always handled lazily by the diff algorithm."},{"ref":"Phoenix.LiveView.Rendered.html","title":"Phoenix.LiveView.Rendered","type":"module","doc":"The struct returned by .leex templates. See a description about its fields and use cases in Phoenix.LiveView.Engine docs."},{"ref":"Phoenix.LiveView.Rendered.html#t:t/0","title":"Phoenix.LiveView.Rendered.t/0","type":"type","doc":""},{"ref":"installation.html","title":"Installation","type":"extras","doc":"Installation While Phoenix LiveView is under heavy development, the installation instructions are likely to change rapidly as well. The instructions below will serve if you are installing the latest stable version from Hex. If you are installing from GitHub to get the latest features, follow the instructions in the README there instead. To start using LiveView, add to your mix.exs and run mix deps.get. If installing from Hex, use the latest version from there: def deps do [ {:phoenix_live_view, &quot;~&gt; 0.4.1&quot;}, {:floki, &quot;&gt;= 0.0.0&quot;, only: :test} ] end If you want the latest features, install from GitHub: def deps do [ {:phoenix_live_view, github: &quot;phoenixframework/phoenix_live_view&quot;}, {:floki, &quot;&gt;= 0.0.0&quot;, only: :test} ] Once installed, update your endpoint&#39;s configuration to include a signing salt. You can generate a signing salt by running mix phx.gen.secret 32. # config/config.exs config :my_app, MyAppWeb.Endpoint, live_view: [ signing_salt: &quot;SECRET_SALT&quot; ] Next, add the LiveView flash plug to your browser pipeline, after :fetch_flash: # lib/my_app_web/router.ex pipeline :browser do ... plug :fetch_flash plug Phoenix.LiveView.Flash end Then add the following imports to your web file in lib/my_app_web.ex: # lib/my_app_web.ex def controller do quote do ... import Phoenix.LiveView.Controller end end def view do quote do ... import Phoenix.LiveView.Helpers end end def router do quote do ... import Phoenix.LiveView.Router end end Next, expose a new socket for LiveView updates in your app&#39;s endpoint module. # lib/my_app_web/endpoint.ex defmodule MyAppWeb.Endpoint do use Phoenix.Endpoint socket &quot;/live&quot;, Phoenix.LiveView.Socket, websocket: [connect_info: [session: @session_options]] # ... end Where @session_options are the options given to plug Plug.Session extracted to a module attribute. Add LiveView NPM dependencies in your assets/package.json. For a regular project, do: { &quot;dependencies&quot;: { &quot;phoenix&quot;: &quot;file:../deps/phoenix&quot;, &quot;phoenix_html&quot;: &quot;file:../deps/phoenix_html&quot;, &quot;phoenix_live_view&quot;: &quot;file:../deps/phoenix_live_view&quot; } } However, if you&#39;re adding phoenix_live_view to an umbrella project, the dependency paths should be modified appropriately: { &quot;dependencies&quot;: { &quot;phoenix&quot;: &quot;file:../../../deps/phoenix&quot;, &quot;phoenix_html&quot;: &quot;file:../../../deps/phoenix_html&quot;, &quot;phoenix_live_view&quot;: &quot;file:../../../deps/phoenix_live_view&quot; } } Then install the new npm dependency. npm install --prefix assets # or `cd assets &amp;&amp; npm install` for Windows users if --prefix doesn&#39;t work If you had previously installed phoenix_live_view and want to get the latest javascript, then force an install. (cd assets &amp;&amp; npm install --force phoenix_live_view) Ensure you have placed a CSRF meta tag inside the &lt;head&gt; tag in your layout (lib/my_app_web/templates/layout/app.html.eex) like so: &lt;%= csrf_meta_tag() %&gt; Enable connecting to a LiveView socket in your app.js file. // assets/js/app.js import {Socket} from &quot;phoenix&quot; import LiveSocket from &quot;phoenix_live_view&quot; let csrfToken = document.querySelector(&quot;meta[name=&#39;csrf-token&#39;]&quot;).getAttribute(&quot;content&quot;); let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, {params: {_csrf_token: csrfToken}}); liveSocket.connect() You can also optionally import the style for the default CSS classes in your app.css file. For a regular project: /* assets/css/app.css */ @import &quot;../../deps/phoenix_live_view/assets/css/live_view.css&quot;; However, if you&#39;re adding phoenix_live_view to an umbrella project, the import link should be modified appropriately: /* assets/css/app.css */ @import &quot;../../../../deps/phoenix_live_view/assets/css/live_view.css&quot;;"}]